/* SPDX-License-Identifier: GPL-2.0-only WITH Linux-syscall-note */
/* Copyright 2022 NXP */

#ifndef __UAPI_DW100_H__
#define __UAPI_DW100_H__

#include <linux/v4l2-controls.h>

/*
 * Check Documentation/userspace-api/media/drivers/dw100.rst for control details.
 */
#define V4L2_CID_DW100_DEWARPING_16x16_VERTEX_MAP (V4L2_CID_USER_DW100_BASE + 1)

#endif
